"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "ScriptLoader", {
    enumerable: true,
    get: function() {
        return ScriptLoader;
    }
});
const _pluginresourceloader = require("@ima/plugin-resource-loader");
const _Events = require("./Events");
class ScriptLoader {
    // IMA.js Window
    _window;
    // IMA.js Dispatcher
    _dispatcher;
    // General-purpose utility for loading resources.
    _resourceLoader;
    // Object of loaded scripts.
    _loadedScripts = {};
    static get $dependencies() {
        return [
            '$Window',
            '$Dispatcher',
            _pluginresourceloader.ResourceLoader
        ];
    }
    constructor(window, dispatcher, resourceLoader){
        this._window = window;
        this._dispatcher = dispatcher;
        this._resourceLoader = resourceLoader;
    }
    /**
   * Load third party script to page.
   *
   * @param url
   * @param template
   * @param force
   * @returns {Promise<{url: string}>}
   */ load(url, template, force = false) {
        if ($Debug) {
            if (!this._window.isClient()) {
                throw new Error(`The script loader cannot be used at the server side. ` + `Attempted to load the ${url} script.`);
            }
        }
        if (url in this._loadedScripts && !force) {
            return this._loadedScripts[url];
        }
        const script = this._createScriptElement();
        this._loadedScripts[url] = this._resourceLoader.promisify(script, template || url).then(()=>this._handleOnLoad(url)).catch(()=>this._handleOnError(url));
        if (template) {
            script.innerHTML = template;
        } else {
            script.async = true;
            script.src = url;
        }
        this._resourceLoader.injectToPage(script);
        if (template) {
            // @ts-expect-error onload is overrided by ResourceLoader promisify, which is kinda weird (TODO rewrite?)
            setTimeout(()=>script.onload(), 0);
        }
        return this._loadedScripts[url];
    }
    /**
   * Creates a new script element and returns it.
   *
   * @returns The created script element.
   */ _createScriptElement() {
        return document.createElement('script');
    }
    /**
   * Handle on load event for script. Resolve load promise and fire LOADED
   * events.
   * @param url
   */ _handleOnLoad(url) {
        const data = {
            url
        };
        this._dispatcher.fire(_Events.Events.LOADED, data, true);
        return data;
    }
    /**
   * Handle on error event for script. Reject load promise and fire LOADED
   * events.
   *
   * @param url
   * @throws Error
   */ _handleOnError(url) {
        const error = new Error(`The ${url} script failed to load.`);
        const data = {
            url,
            error
        };
        this._dispatcher.fire(_Events.Events.LOADED, data, true);
        throw error;
    }
}

//# sourceMappingURL=ScriptLoader.js.map